/* Llauncher
 *
 *      a snazzy external application launcher
 *	Created for the Winter 06 iPL contest.
 *	(just after courtc fixed the vt switching. YAY!)
 *      
 *      (c)2006/2007 Scott Lawrence
 *	   yorgle@gmail.com
 */

/*
 * Copyright (C) 2006/2007 Scott Lawrence
 *
 * WARNING: May contain trace amounts of monkeys.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#define L_VERS	"0005"
/*
 *               0005
 *  2007-01-04 - Built and tested for iPod
 *		 file browser launcher for .cfg and .txt files
 *		 0004
 *  2007-01-03 - added the version number to the titlebar.  How meta!
 *               Added zoom and slide animations
 *               0002/0003
 *  2007-01-03 - minor tweaks and bugfixes
 *  2007-01-02 - Slide animation, user prefs/settings, color transitions complete
 *		 0001
 *  2007-01-02 - got the first _draw and navigation working, basic functionality is complete
 *  2007-01-01 - Settled on INI format for files, wrote INI-ITEM converter
 *  2006-12-31 - Started, experimented with a few different parsers/files
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h> 	/* for O_RDONLY, open(), etc */
#include <unistd.h>	/* chdir() */
#include "pz.h"
#include "ini.h"	/* for the conf file */
#include "llist.h"
#include "search.h"	/* file path searching */


/* the indexes for the transitions - must match transition_options[] below */
#define LLAUNCHER_TRANS_CUT		(0)
#define LLAUNCHER_TRANS_SLIDE		(1)
#define LLAUNCHER_TRANS_SLIDEOVER	(2)
#define LLAUNCHER_TRANS_SLIDEZOOM	(3)	
#define LLAUNCHER_TRANS_ZOOM		(4)

/* these are keys into our config file, for the settings */
#define CFG_TRANSITION (1)
#define CFG_SPEED      (2)
#define CFG_FULLSCREEN (3)
#define CFG_NOTEXT     (4)
#define CFG_SKIPOVER   (5)

/* and here are the options for the menu settings */
const char * transition_options[] = {
	"Cut",
	"Slide",
	"Slide Over",
	"Slide Zoom",
	"Zoom",
	0
};

const char * speed_options[] = {
	"Very Fast",
	"Fast",
	"Medium",
	"Slow",
	"Slower",
	"Slowerest",
	"Very Slowerest",
	0
};

const char * noyes_options[] = { "No", "Yes", 0 };
const char * yesno_options[] = { "Yes", "No", 0 };


/* these are used for the animation - lglob.direction */
#define DIRECTION_LEFT	(0)
#define DIRECTION_RIGHT	(1)
#define DIRECTION_UP	(2)
#define DIRECTION_DOWN	(3)

/*
** the globals for the module
*/
typedef struct _llauncher_globals {
	/* pz meta stuff */
	PzModule * module;
	PzWindow * window;
	PzWidget * widget;

	/* screen stuff */
	int w, h;

	/* various helpers */
	int paused;
	int timer;
	int frames;
	int isMono;

	/* settings */
	PzConfig *config;
	int fullScreen;		/* go fullscreen, without the title bar? */
	int transition;		/* which transition to use */
	int speed;		/* transition speed - small = fast */
	int noText;		/* turn off all text completely */
	int skipOver;		/* = let transitions skip over eachother */

	/* for animations */
	int transitionFrame;
	int direction;

	/* for the file - filled in at module use time */
	INIStruct * parsedFile;
	LLAUNCHER_ITEM * menuHeirarchy;

	/* and for runtime */
	LLAUNCHER_ITEM * activeMenu;
	LLAUNCHER_ITEM * activeMenuPrevious;
	LLAUNCHER_ITEM * hover;
	LLAUNCHER_ITEM * hoverPrevious;
} llauncher_globals;

/* and define an instance of the above... */
static llauncher_globals lglob;

#define LLAUNCHER_TIMER		(10) /* ms between screen updates */


extern void pz_exec( const char * );

/*
** _launch	- a wrapper for pz_exec which adds a working directory
**                -- perhaps this should be put into browser.c?
*/
static void llauncher_exec( char * working_dir, char * command )
{
	int lwd = 0;

	if( working_dir ) {
		lwd = open( ".", O_RDONLY );
		chdir( working_dir );
	}

	pz_exec( command );

	if( working_dir ) {
		fchdir( lwd );
		close( lwd );
	}
}


/*
** mix_colors 	- a helper to mix color a with color b, based on curr/max
*/
static ttk_color mix_colors( ttk_color a, ttk_color b, int curr, int max )
{
	int r1, g1, b1, r2, g2, b2;

	ttk_unmakecol( a, &r1, &g1, &b1 );
	ttk_unmakecol( b, &r2, &g2, &b2 );

	return( ttk_makecol(
			(r1 * curr / max) + (r2 * (max-curr) / max ),
			(g1 * curr / max) + (g2 * (max-curr) / max ),
			(b1 * curr / max) + (b2 * (max-curr) / max ) ));
}


/*
** _launch	- run the specific item 
*/
static void llauncher_launch( LLAUNCHER_ITEM * li )
{
	if( !li || !li->command ) return;

	//printf( "Launch %s\n", li->command );

	llauncher_exec( li->workingDirectory, li->command );
}



/*
** _outline_text	- draw the text with a bg outline
*/
void llauncher_outline_text( ttk_surface srf, ttk_font font,
				int x, int y, ttk_color bg, ttk_color fg,
				const char *str )
{
	ttk_text( srf, font, x-1, y, bg, str );
	ttk_text( srf, font, x+1, y, bg, str );
	ttk_text( srf, font, x, y-1, bg, str );
	ttk_text( srf, font, x, y+1, bg, str );
	ttk_text( srf, font, x, y, fg, str );
}


/*
** _drawItem 	- draw out the item to the screen
*/
void llauncher_drawItem( ttk_surface srf, LLAUNCHER_ITEM * li, 
				int offx, int offy, float scale )
{
	int w, h;
	int textw, texth;
	ttk_surface blitThis;

	if( !li || !lglob.activeMenu->title || !li->icon )
		return;

	textw = ttk_text_width( ttk_textfont, lglob.activeMenu->title );
	texth = ttk_text_height( ttk_textfont );

	if( scale == 1.0 ) {
		blitThis = li->icon;
	} else {
		blitThis = ttk_scale_surface( li->icon, scale );
	}

	ttk_surface_get_dimen( blitThis, &w, &h );
	ttk_blit_image( blitThis, srf,
			offx + ( (lglob.w-w)>>1 ), 
			offy + ( (lglob.h-h)>>1 ) );
		
	if( scale != 1.0 ) {
		ttk_free_image( blitThis );
	}
}


/*
** drawAnimation_slide	- slides one over another, etc
*/
int llauncher_drawAnimation_slide( ttk_surface srf,
				LLAUNCHER_ITEM * old, LLAUNCHER_ITEM * new,
				int frame, int nframes, int solid )
{
	ttk_color bg = lglob.activeMenu->bg;
	//ttk_color fg = lglob.activeMenu->fg;

	int x = lglob.w * frame / nframes;
	int y = lglob.h * frame / nframes;

	int texth = 0;
	int textw = 0;

	if( lglob.activeMenu->title && !lglob.noText ) {
		texth = ttk_text_height( ttk_textfont );
		textw = ttk_text_width( ttk_textfont, lglob.activeMenu->title );
	}

	switch( lglob.direction ) {
		case( DIRECTION_LEFT ):
			/* icon */
			x = lglob.w-x;
			ttk_fillrect( srf, 0, 0, lglob.w, lglob.h, bg );
			llauncher_drawItem( srf, old, solid?0:x, 0, 1.0 );
			llauncher_drawItem( srf, new, x-lglob.w, 0, 1.0 );

			/* title */
			if( lglob.activeMenu->title && !lglob.noText ) {
				llauncher_outline_text( srf, ttk_textfont,
					(lglob.w - textw)>>1, 5,
					lglob.activeMenu->bg, lglob.activeMenu->fg,
					lglob.activeMenu->title );
			}
			break;

		case( DIRECTION_RIGHT ):
			/* icon */
			ttk_fillrect( srf, 0, 0, lglob.w, lglob.h, bg );
			llauncher_drawItem( srf, old, solid?0:x-lglob.w, 0, 1.0 );
			llauncher_drawItem( srf, new, x, 0, 1.0 );

			/* title */
			if( lglob.activeMenu->title && !lglob.noText ) {
				llauncher_outline_text( srf, ttk_textfont,
					(lglob.w - textw)>>1, 5,
					lglob.activeMenu->bg, lglob.activeMenu->fg,
					lglob.activeMenu->title );
			}
			break;

		case( DIRECTION_UP ):
			bg = mix_colors( lglob.activeMenuPrevious->bg,
					 lglob.activeMenu->bg,
					 frame, nframes );
			ttk_fillrect( srf, 0, 0, lglob.w, lglob.h, bg );
			llauncher_drawItem( srf, new, 0, y, 1.0 );
			llauncher_drawItem( srf, old, 0, y-lglob.h, 1.0 );
			break;

		case( DIRECTION_DOWN ):
			bg = mix_colors( lglob.activeMenuPrevious->bg,
					 lglob.activeMenu->bg,
					 frame, nframes );
			ttk_fillrect( srf, 0, 0, lglob.w, lglob.h, bg );
			y = lglob.h-y;
			llauncher_drawItem( srf, old, 0, y, 1.0 );
			llauncher_drawItem( srf, new, 0, y-lglob.h, 1.0 );
			break;

		default:
			return 0;
	}
	return( frame-1 );
}


/*
** drawAnimation_zoom	- the zoom animations
*/
int llauncher_drawAnimation_zoom( ttk_surface srf,
				LLAUNCHER_ITEM * old, LLAUNCHER_ITEM * new,
				int frame, int nframes, int slide )
{
	float scale = (float)frame/(float)nframes;
	int xoffs = 0;
	int nxoffs = 0;

	if( slide ) {
		xoffs = (lglob.w/3) * frame / nframes;
		nxoffs = -(lglob.w/3) + xoffs;
	}

	
	if(    (lglob.direction == DIRECTION_UP)
	    || (lglob.direction == DIRECTION_DOWN ) )
	{
		return llauncher_drawAnimation_slide( 
				srf, old, new, frame, nframes, 0 );
	}


	if( lglob.direction == DIRECTION_RIGHT ) {
		llauncher_drawItem( srf, new, xoffs, 0, 1.0-scale );
		llauncher_drawItem( srf, old, nxoffs, 0, scale );
	} else {
		llauncher_drawItem( srf, new, -xoffs, 0, 1.0-scale );
		llauncher_drawItem( srf, old, -nxoffs, 0, scale );
	}

	return( frame-1 );
}


/*
** _drawAnimation	- the valve for all of the various animation handlers
*/
int llauncher_drawAnimation( ttk_surface srf,
				LLAUNCHER_ITEM * old, LLAUNCHER_ITEM * new,
				int frame, int nframes )
{
	
	if( !old || !new ) {
		return 0;
	}

	switch( lglob.transition ) {

	case( LLAUNCHER_TRANS_CUT ):
		return -1;	/* -1 signifies "done" and "continue to draw" */

	case( LLAUNCHER_TRANS_SLIDE ):
		return llauncher_drawAnimation_slide( 
				srf, old, new, frame, nframes, 0 );

	case( LLAUNCHER_TRANS_SLIDEOVER ):
		return llauncher_drawAnimation_slide( 
				srf, old, new, frame, nframes, 1 );

	case( LLAUNCHER_TRANS_SLIDEZOOM ):
		return llauncher_drawAnimation_zoom( 
				srf, old, new, frame, nframes, 1 );

	case( LLAUNCHER_TRANS_ZOOM ):
		return llauncher_drawAnimation_zoom( 
				srf, old, new, frame, nframes, 0 );

	default:
		return( 0 ); /* 0 signifies "done" and "return" */
	}

	return( frame-1 );
}


/*
** _draw - draw a frame to the screen
*/
static void llauncher_draw( PzWidget *widget, ttk_surface srf )
{
	int textw = 0;
	int texth = 0; 
	short triX[4];
	short triY[4];

	if( !lglob.activeMenu ) {
		return;
	}

	lglob.frames++;


	/* first, clear the screen */
	ttk_color bg = lglob.activeMenu->bg;
	ttk_color fg = lglob.activeMenu->fg;

	ttk_fillrect( srf, 0, 0, lglob.w, lglob.h, bg );


	/* draw animation */
	if( lglob.transitionFrame > 0 ) {
		lglob.transitionFrame = llauncher_drawAnimation( srf,
				lglob.hoverPrevious, lglob.hover,
				lglob.transitionFrame, lglob.speed );

		if( lglob.transitionFrame < 0 ) {
			lglob.transitionFrame = 0;
		} else {
			return;
		}
	}


	/* draw the icon, title, text */
	if( lglob.hover->flags & LLAUNCHER_FLAGS_ICON ) {
		llauncher_drawItem( srf, lglob.hover, 0, 0, 1.0 );
	}

	/* draw the header */
	texth = ttk_text_height( ttk_textfont );
	if( lglob.activeMenu->title && !lglob.noText ) {
		textw = ttk_text_width( ttk_textfont, lglob.activeMenu->title );
		llauncher_outline_text( srf, ttk_textfont,
			(lglob.w - textw)>>1, 5,
			lglob.activeMenu->bg, lglob.activeMenu->fg,
			lglob.activeMenu->title );
	}

	/* bottom text */
	if(    lglob.hover->title 
	    && (lglob.hover->flags & LLAUNCHER_FLAGS_TEXT) 
	    && !lglob.noText ) {
		textw = ttk_text_width( ttk_textfont, lglob.hover->title );
		llauncher_outline_text( srf, ttk_textfont,
			(lglob.w - textw)>>1,
			lglob.h - ( (texth) +3 )*2,
			lglob.hover->parent->bg, lglob.hover->parent->fg,
			lglob.hover->title );
	}

	/* bottom infotext */
	if( lglob.hover->info && !lglob.noText ) {
		textw = ttk_text_width( ttk_textfont, lglob.hover->info );
		llauncher_outline_text( srf, ttk_textfont,
			(lglob.w - textw)>>1, 
			lglob.h - texth - 3,
			lglob.hover->parent->bg, lglob.hover->parent->fg,
		        lglob.hover->info );
	}
	
	/* draw arrows - in stereo where available */
	if( lglob.activeMenu->arrowW && lglob.activeMenu->arrowH ) {
		if( lglob.hover->previous ) {
			triY[0] = lglob.h/2;
			triY[1] = (lglob.h/2)-lglob.activeMenu->arrowH;
			triY[2] = (lglob.h/2)+lglob.activeMenu->arrowH;
			triX[0] = 5;
			triX[1] = 5 + lglob.activeMenu->arrowW;
			triX[2] = 5 + lglob.activeMenu->arrowW; 

			ttk_fillpoly( srf, 3, triX, triY, fg );
			ttk_aapoly( srf, 3, triX, triY, fg );
		 }

		if( lglob.hover->next ) {
			triY[0] = lglob.h/2;
			triY[1] = (lglob.h/2)-lglob.activeMenu->arrowH;
			triY[2] = (lglob.h/2)+lglob.activeMenu->arrowH;
			triX[0] = lglob.w - 5;
			triX[1] = lglob.w - (5 + lglob.activeMenu->arrowW);
			triX[2] = lglob.w - (5 + lglob.activeMenu->arrowW);

			ttk_fillpoly( srf, 3, triX, triY, fg );
			ttk_aapoly( srf, 3, triX, triY, fg );
		}
	}
}



/*
** _timer - we got a timetick
*/
static void llauncher_timer( int nticks )
{
}


/*
** _session_end - we're closing, clean up allocated ram and such
*/
static void llauncher_session_end( void )
{
	pz_widget_set_timer( lglob.widget, 0 );
	llauncher_delete( lglob.menuHeirarchy );
	lglob.menuHeirarchy = NULL;
	ini_Destroy( lglob.parsedFile );
	lglob.parsedFile = NULL;

	lglob.activeMenu = NULL;
	lglob.activeMenuPrevious = NULL;
	lglob.hover = NULL;
	lglob.hoverPrevious = NULL;
}


/*
** _destroy - we're shutting down podzilla! Aiee!
*/
static void llauncher_destroy( void ) 
{
}


static void llauncher_rightSpin( void )
{
	/* if we're animating, and skipOver is disabled, then return */
	if( lglob.transitionFrame && !lglob.skipOver ) return;

	if( lglob.hover->next )
	{
		lglob.transitionFrame = lglob.speed;
		lglob.direction = DIRECTION_RIGHT;
		lglob.hoverPrevious = lglob.hover;
		lglob.hover = lglob.hover->next;
	}
}

static void llauncher_leftSpin( void )
{
	/* if we're animating, and skipOver is disabled, then return */
	if( lglob.transitionFrame && !lglob.skipOver ) return;

	if( lglob.hover->previous ) {
		lglob.transitionFrame = lglob.speed;
		lglob.direction = DIRECTION_LEFT;
		lglob.hoverPrevious = lglob.hover;
		lglob.hover = lglob.hover->previous;
	}
}

/*
** _event - we've gotten a podzilla event (user input, timer, etc)
*/
static int llauncher_event( PzEvent *ev )
{
	switch (ev->type) {
	case PZ_EVENT_SCROLL:
		if( !lglob.paused ) {
			TTK_SCROLLMOD( ev->arg, 10 );
			if( ev->arg > 0 ) {
				/* scroll event */
				llauncher_rightSpin();
			} else {
				llauncher_leftSpin();
			}
			ev->wid->dirty = 1;
		}
		break;

	case PZ_EVENT_BUTTON_DOWN:
		switch( ev->arg ) {
		case( PZ_BUTTON_HOLD ):
			lglob.paused = 1;
			ev->wid->dirty = 1;
			break;
		}
		break;

	case PZ_EVENT_BUTTON_UP:
		switch( ev->arg ) {
		case( PZ_BUTTON_MENU ):

			if( lglob.activeMenu == lglob.menuHeirarchy ) {
				/* if we're already at the top, exit out... */
				llauncher_session_end();
				pz_close_window (ev->wid->win);
			} else {
				/* otherwise, go up one menu */
				lglob.transitionFrame = lglob.speed;
				lglob.direction = DIRECTION_DOWN;
				lglob.hoverPrevious = lglob.hover;
				lglob.hover = lglob.hover->parent;
				lglob.activeMenuPrevious = lglob.activeMenu;
				lglob.activeMenu = lglob.activeMenu->parent;
			}
			ev->wid->dirty = 1;
			break;

		case( PZ_BUTTON_HOLD ):
			lglob.paused = 0;
			ev->wid->dirty = 1;
			break;

		case( PZ_BUTTON_PREVIOUS ):
			ev->wid->dirty = 1;
			break;

		case( PZ_BUTTON_NEXT ):
			ev->wid->dirty = 1;
			break;

		case( PZ_BUTTON_PLAY ):
			ev->wid->dirty = 1;
			break;

		case( PZ_BUTTON_ACTION ):
			if( lglob.activeMenu->items ) {
				/* do submenu */
				if( lglob.hover->command ) {
					llauncher_launch( lglob.hover );
				} else {
					lglob.transitionFrame = lglob.speed;
					lglob.direction = DIRECTION_UP;
					lglob.activeMenuPrevious = lglob.activeMenu;
					lglob.activeMenu = lglob.hover;
					lglob.hoverPrevious = lglob.hover;
					lglob.hover = lglob.activeMenu->items;
				}
			} else {
				/* it should never get in here */
				llauncher_launch( lglob.hover->items );
			}
			ev->wid->dirty = 1;
			break;
		}
		break;

	case PZ_EVENT_DESTROY:
		llauncher_destroy();
		break;

	case PZ_EVENT_TIMER:
		lglob.timer++;
		llauncher_timer( lglob.timer );
		ev->wid->dirty = 1;
		break;
	}

	return 0;
}


/*
** _init_globals - initialization and window/parse tree setup routines
*/
static void llauncher_init_globals( void )
{
	lglob.module = NULL;
	lglob.window = NULL;
	lglob.widget = NULL;
	lglob.w = ttk_screen->w - ttk_screen->wx;
	lglob.h = ttk_screen->h - ttk_screen->wy;
	lglob.paused = 0;
	lglob.timer = 0;
	lglob.frames = 0;
	lglob.isMono = ( ttk_screen->bpp < 16 )?1:0;
	lglob.parsedFile = NULL;
	lglob.menuHeirarchy = NULL;

	lglob.fullScreen = 0;
	lglob.transition = LLAUNCHER_TRANS_CUT;
	lglob.speed = 10;
	lglob.noText = 0;
}


/* 
**  _load_config - load in the configuration file
*/
int llauncher_load_config( const char * filename )
{
	char *validPath = llauncher_resolve_filepath( llauncher_searchPath, (char *)filename );

	if( validPath ) {
                lglob.parsedFile = ini_ParseFile( validPath );
                if( lglob.parsedFile )
                {
                        //ini_Dump( lglob.parsedFile );
			lglob.menuHeirarchy = llauncher_buildMenus( lglob.parsedFile );
			lglob.activeMenu = lglob.menuHeirarchy;
			lglob.hover = lglob.activeMenu->items;
			free( validPath );

			/* now add in the settings for the system */
			lglob.transition = pz_get_int_setting( lglob.config, CFG_TRANSITION );
			lglob.speed = (pz_get_int_setting( lglob.config, CFG_SPEED ) + 1) * 4;
			lglob.fullScreen = pz_get_int_setting( lglob.config, CFG_FULLSCREEN );
			lglob.noText = pz_get_int_setting( lglob.config, CFG_NOTEXT );
			lglob.skipOver = pz_get_int_setting( lglob.config, CFG_SKIPOVER );
			pz_save_config( lglob.config ); /* just in case... */

                        return( 0 );
                } 
		free( validPath );
	}
	return( 1 );
}

/*
** _newLlauncherWindow - create the new window and associated stuff
*/
PzWindow *llauncher_newLlauncherWindow( const char * filename )
{
	llauncher_init_globals();
	if( llauncher_load_config( filename ) )
	{
		pz_error( "Unable to load conf file!" );
		return NULL;
	}

	/* setup the window */
	lglob.window = pz_new_window( "Llauncher " L_VERS, PZ_WINDOW_NORMAL );

	/* adjust for fullscreen? */
	if( lglob.fullScreen )
	{
		ttk_window_hide_header( lglob.window );
		lglob.window->x = 0;
		lglob.window->y = 0;
		lglob.window->w = ttk_screen->w;
		lglob.window->h = ttk_screen->h;
		lglob.w = ttk_screen->w;
		lglob.h = ttk_screen->h;
	}

	lglob.widget = pz_add_widget( lglob.window, 
				llauncher_draw, llauncher_event );

	/* adjust for persistence - ie, no clear between frames */
	if( 0 ) 
	{
		lglob.widget->w = lglob.widget->h = 0;
	}

	/* setup a timer */
	pz_widget_set_timer( lglob.widget, LLAUNCHER_TIMER );

	return( pz_finish_window( lglob.window ));
}

PzWindow *llauncher_new_window( void )
{
	return( llauncher_newLlauncherWindow( "llaunch.txt" ));
}

/*
** here's our hooks into the filebrowser
*/
static ttk_menu_item llauncher_fbx;

int llauncher_openable( const char * filename )
{
	if( !filename )
		return( 0 );

	if( strlen( filename ) < 4 )
		return( 0 );

	if( !strcmp( ".txt", filename + strlen( filename ) -4 ))
		return( 1 );

	if( !strcmp( ".cfg", filename + strlen( filename ) -4 ))
		return( 1 );

	return( 0 );
}

PzWindow * llauncher_open_handler( ttk_menu_item * item )
{
	return( llauncher_newLlauncherWindow( item->data ));
}


/*
**  our module's init function...
*/
void llauncher_module_init() 
{
	/* internal module name */
	lglob.module = pz_register_module( "llauncher", llauncher_destroy );

	/* settings */
	lglob.config = pz_load_config( pz_module_get_cfgpath( 
			lglob.module, "llauncher.conf" ));

	pz_menu_add_setting( "/Settings/Llauncher/Transition", CFG_TRANSITION, 
			lglob.config, transition_options );
	pz_menu_add_setting( "/Settings/Llauncher/Speed", CFG_SPEED, 
			lglob.config, speed_options );
	pz_menu_add_setting( "/Settings/Llauncher/Full Screen", CFG_FULLSCREEN, 
			lglob.config, noyes_options );
	pz_menu_add_setting( "/Settings/Llauncher/Skip Over", CFG_SKIPOVER, 
			lglob.config, noyes_options );
	pz_menu_add_setting( "/Settings/Llauncher/Hide Text", CFG_NOTEXT,
			lglob.config, noyes_options );

	/* menu location */
	(void)pz_menu_add_action( "/Extras/Llauncher", llauncher_new_window );

	/* and create the browser hook */
	llauncher_fbx.name = N_( "Open with Llauncher" );
	llauncher_fbx.makesub = llauncher_open_handler;
	pz_browser_add_action( llauncher_openable, &llauncher_fbx );
}



PZ_MOD_INIT (llauncher_module_init)
