/* Llist
 *
 *  llauncher llist mmanager
 *      
 *      (c)2006 Scott Lawrence
 *	   yorgle@gmail.com
 */

/*
 * Copyright (C) 2006/2007 Scott Lawrence
 *
 * WARNING: May contain trace amounts of monkeys.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include "pz.h"
#include "ini.h"	/* for the conf file */
#include "llist.h"
#include "search.h"


/* _new 	- create a new, blank node 
 */
LLAUNCHER_ITEM * llauncher_new()
{
	LLAUNCHER_ITEM * li = (LLAUNCHER_ITEM *)malloc( sizeof(LLAUNCHER_ITEM) );
	if( !li ) return NULL;

	li->menuGroup = NULL;
	li->title = NULL;
	li->info = NULL;
        li->bg = ttk_ap_get( "menu.bg" )->color;
        li->fg = ttk_ap_get( "menu.fg" )->color;
	li->image = NULL;
	li->flags = LLAUNCHER_FLAGS_NONE;
	li->arrowW = li->arrowH = 10;

	li->workingDirectory = NULL;
	li->command = NULL;

	li->items = NULL;
	li->next = NULL;

	li->previous = NULL;
	li->parent = NULL;
	return( li );
}


/* _delete	- free up a node and his children, recursively
 */
void llauncher_delete( LLAUNCHER_ITEM * li )
{
	if( !li ) return;

	if( li->title )             free( li->title );
	if( li->info )              free( li->info );
	if( li->command )           free( li->command );
	if( li->menuGroup )         free( li->menuGroup );
	if( li->workingDirectory )  free( li->workingDirectory );
	if( li->image )             ttk_free_image( li->image );
	llauncher_delete( li->items );
	llauncher_delete( li->next );
	free( li );
}


/* _menuExists	- if a group exists, return 1, otherwise 0
 */
int llauncher_menuExists( LLAUNCHER_ITEM * tree, char * groupName )
{
	LLAUNCHER_ITEM *tmp = tree;
	while( tmp )
	{
		if( tmp->items ) {
			if( llauncher_menuExists( tmp->items, groupName )) return 1;
		}
		if( !strcmp( groupName, tmp->menuGroup )) return 1;
		tmp = tmp->next;
	}
	return 0;
}


/* _addMenu	- recursively add the menu items for the specified menuGroup
 */
LLAUNCHER_ITEM * llauncher_addMenu( 
			LLAUNCHER_ITEM * tree,
			INIStruct * parsedFile,
			char * menuGroup,
			int level )
{
	LLAUNCHER_ITEM * li = NULL;
	LLAUNCHER_ITEM * li2 = NULL;
	LLAUNCHER_ITEM * subitems = NULL;
	LLAUNCHER_ITEM * si2 = NULL;
	int x;
	int r,g,b;
	int w,h;

	/* we have a few bits of error detection here:
		- first, we make sure that the user doesn't have any recursion loops
			- basically, we just increment a "level" counter
			- when this reaches 16, we dump a warning.
			- 16 is arbitrary, but reasonably deep

		- second, we make sure that the item doesn't already exist in the
		  tree.  This should never happen, but we all know that those are
		  famous last words... much like "what could possibly go wrong?" etc.

		- thirdst, we look for the configuration group in the ini file.
		  	- if there was none, we bail.
	*/ 

	if( level > 16 ) { /* arbitrary, but reasonable number */
		fprintf( stderr, "Llauncher: %d levels!  Recursion loop with \"%s\"?\n",
				level, menuGroup );
		return( NULL );
	}
	if( llauncher_menuExists( tree, menuGroup ))
	{
		fprintf( stderr, "Llauncher: \"%s\": Item already exists!\n", menuGroup );
		return( NULL );	
	}
	if( !ini_GroupExists( parsedFile, menuGroup ))
	{
		fprintf( stderr, "Llauncher: Unable to find configuration group \"%s\"\n",
				menuGroup );
		return( NULL );
	}

	li = llauncher_new();
	if( !li )
	{
		fprintf( stderr, "Llauncher: Malloc Error\n" );
		return( NULL );
	}

	/* now, create the new node, and figure out what goes into it */
	li->menuGroup = strdup( menuGroup );
	if( ini_KeyExists( parsedFile, menuGroup, "TITLE" )) {
		li->title = strdup( ini_Query( parsedFile, menuGroup, "TITLE" ));
	}
	if( ini_KeyExists( parsedFile, menuGroup, "INFO" )) {
		li->info = strdup( ini_Query( parsedFile, menuGroup, "INFO" ));
	}
	if( ini_KeyExists( parsedFile, menuGroup, "ARROWSIZE" )) {
		w = ini_QueryArrayInt( parsedFile, menuGroup, "ARROWSIZE", 0 );
		h = ini_QueryArrayInt( parsedFile, menuGroup, "ARROWSIZE", 1 );
		li->arrowW = w;
		li->arrowH = h;
	}
	/* selectable icon for the list */
	if( ini_KeyExists( parsedFile, menuGroup, "ICON" )) {
		char * tfn = llauncher_resolve_filepath(
                                        llauncher_searchPath, 
					ini_Query( parsedFile, menuGroup, "ICON" ));

		if( tfn ) {
			li->icon = ttk_load_image( tfn );
			free( tfn );
		}
	}
	/* this is only valid for menu items, but whatever... */
	if( ini_KeyExists( parsedFile, menuGroup, "BGIMAGE" )) {
		char * tfn = llauncher_resolve_filepath(
                                        llauncher_searchPath, 
					ini_Query( parsedFile, menuGroup, "BGIMAGE" ));

		if( tfn ) {
			li->image = ttk_load_image( tfn );
			free( tfn );
		}
	}
	/* set the colors - these are defaulted to window.fg/bg if not defined */
	if( ini_KeyExists( parsedFile, menuGroup, "BGCOLOR" )) {
		r = ini_QueryArrayInt( parsedFile, menuGroup, "BGCOLOR", 0 );
		g = ini_QueryArrayInt( parsedFile, menuGroup, "BGCOLOR", 1 );
		b = ini_QueryArrayInt( parsedFile, menuGroup, "BGCOLOR", 2 );
		li->bg = ttk_makecol( r, g, b );
	}
	if( ini_KeyExists( parsedFile, menuGroup, "FGCOLOR" )) {
		r = ini_QueryArrayInt( parsedFile, menuGroup, "FGCOLOR", 0 );
		g = ini_QueryArrayInt( parsedFile, menuGroup, "FGCOLOR", 1 );
		b = ini_QueryArrayInt( parsedFile, menuGroup, "FGCOLOR", 2 );
		li->fg = ttk_makecol( r, g, b );
	}

	if( ini_KeyExists( parsedFile, menuGroup, "FLAGS" )) {
		char * val = ini_Query( parsedFile, menuGroup, "FLAGS" );
		if( val ) {
			if( !strcmp( val, "icon" )) li->flags |= LLAUNCHER_FLAGS_ICON;
			if( !strcmp( val, "text" )) li->flags |= LLAUNCHER_FLAGS_TEXT;
			if( !strcmp( val, "icontext" ))
				li->flags |= LLAUNCHER_FLAGS_ICON + LLAUNCHER_FLAGS_TEXT;
		}
	}
	/* try to figure it out, if there were no flags set */
	if( ((li->flags & LLAUNCHER_FLAGS_ICON)==0) &&
	    ((li->flags & LLAUNCHER_FLAGS_TEXT)==0) ) {
		if( li->title ) li->flags |= LLAUNCHER_FLAGS_TEXT;
		if( li->icon ) li->flags |= LLAUNCHER_FLAGS_ICON;
	}

	/* CMD is required for launchables, add it in, set the flags */
	if( ini_KeyExists( parsedFile, menuGroup, "CMD" )) {
		li->command = strdup( ini_Query( parsedFile, menuGroup, "CMD" ));
	}
	if( ini_KeyExists( parsedFile, menuGroup, "DIR" )) {
		li->workingDirectory = strdup( ini_Query( parsedFile, menuGroup, "DIR" ));
	}

	/* set the parent */
	li->parent = tree;
	
	/* the following will recursively walk down the list of items 0,1,2,etc,
           attaching the subm menus... */
	for( x = 0 ; ini_IntKeyExists( parsedFile, menuGroup, x ) ; x++ )
	{
		li2 = llauncher_addMenu( li, parsedFile, 
			ini_Query_IntKey( parsedFile, menuGroup, x ), level+1);

		/* make sure the subitem was valid. */
		if( li2 ) {
			if( subitems == NULL ) { /* no items, make it the new head */
				subitems = li2;

			} else { /* hay items, make it the end */
				si2 = subitems;
				while( si2->next ) {
					si2 = si2->next;
				}
				si2->next = li2;
				li2->previous = si2;
			}
		}
	}

	/* this.items linkages are made here (above) */
	/* this.next  linkages are made in the parent (the "above" in the caller) */
	li->items = subitems; /* add on the submenus, or NULL */

	/* send "this" back to the parent caller */
	return( li );
}



/* _buildMenus	- build the menus, starting with "ROOT"
 */
LLAUNCHER_ITEM * llauncher_buildMenus( INIStruct * parsedFile )
{
	LLAUNCHER_ITEM * tree = NULL;

	if( !parsedFile ) return NULL;

	/* start with the ROOT - go from there */
	tree = llauncher_addMenu( tree, parsedFile, "ROOT", 0 );

	return( tree );
}
